layout = {}
uiState = {}

local debugPrint = function(string)
	print(string)
end


function layout.canvasAndColor(container, x, y, simulate)
	if simulate then return layout.handle(container[1], x, y, simulate) end
	color_push()
	love.graphics.setCanvas(container.canvas)
	x, y = layout.handle(container[1], x, y, simulate)
	love.graphics.setCanvas()
	color_pop()
	return x, y
end


function layout.frame(container, x, y, simulate)
	assert_num("layout.frame", 2, x, y)
	x2, y2 = layout.handle(container[1], x +1, y +1, simulate)
	if not simulate then
		love.graphics.rectangle("line", x, y, x2 -x, y2 -y)
	end
	return x2+1, y2+1
end



function layout.cursorSelect(container, x, y, simulate)
	x2, y2 = layout.handle(container[1], x, y, simulate)
	if not simulate then
		table.insert(uiState, { identifier = container.identifier, kind=container.kind,
			x=x, x2=x2, y=y, y2=y2 })
	end
	return x2, y2
end


function layout.linewidth(container, x, y, simulate)
	if simulate then return layout.handle(container[1], x, y, simulate) end
	assert(container.width, "layout.linewidth (container) no container.width provided!")
	local oldWidth = love.graphics.getLineWidth()
	love.graphics.setLineWidth(container.width)
	x, y = layout.handle(container[1], x, y, simulate)
	love.graphics.setLineWidth(oldWidth)
	return x, y
end


function layout.spacer(container, x, y, simulate)
	assert(container.width or container.height, "No arguments for layout.spacer!")
	assert(container.width == container.width,
		"Argument container.width to layout.spacer must not be NaN!")
	assert(container.height == container.height,
		"Argument container.height to layout.spacer must not be NaN!")
	local width = container.width or 0
	local height = container.height or 0
	if container[1] then
		return layout.handle(container[1], x + width, y + height, simulate)
	else
		return x + width, y + height
	end
end


function layout.drawTexture(container, x, y, simulate)
	texture = container.texture
	if not simulate then
		love.graphics.draw(texture, x, y)
	end
	return x + texture:getWidth(), y + texture:getHeight()
end


function layout.color(container, x, y, simulate)
	if simulate then return layout.handle(container[1], x, y, simulate) end
	assert_num("layout.color (container[r,g,b])", 3, container.r, container.g, container.b)
	assert_num_or_nil("layout.color (container[a])", 1, container.a)
	color_push()
	love.graphics.setColor(container.r, container.g, container.b, container.a or 1)
	x, y = layout.handle(container[1], x, y, simulate)
	color_pop()
	return x, y
end


function layout.rect(container, x, y, simulate)
	assert_num("layout.rect", 2, x, y)
	assert_num("layout.rect (container)", 2, container.width, container.height)
	if not simulate then
		if container.color then
			color_push()
			love.graphics.setColor(container.color.r, container.color.g, container.color.b, container.color.a or 1)
		love.graphics.rectangle(container.fill or "fill", x, y, container.width, container.height)
			color_pop()
		else
			love.graphics.rectangle(container.fill or "fill", x, y, container.width, container.height)
		end
	end
	return x + container.width, y + container.height
end


function layout.overlayRect(container, x, y, simulate)
	assert_num("layout.overlayRect", 2, x, y)
	winx, winy = layout.handle(container[1], x, y, simulate)
	width = winx -x
	height = winy -y
	if not simulate then
		love.graphics.rectangle(container.fill or "fill", x, y, width, height)
	end
	return x + width, y + height
end



function layout.label(container, x, y, simulate)
	local text = love.graphics.newText( ui.font, container.text )
	if not text then text = "Unset text!!" end
	if not simulate then
		love.graphics.draw(text, x, y)
	end
	return x + text:getWidth(), y + text:getHeight()
end


function layout.layer(container, x, y, simulate)
	local ret1 = {}
	local ret2 = {}
	ret1.x, ret1.y = layout.handle(container[1], x, y, simulate)
	ret2.x, ret2.y  = layout.handle(container[2], x, y, simulate)
	return math.max(ret1.x,ret2.x), math.max(ret1.y, ret2.y)
end


function layout.horizontal(container, x, y, simulate)
	local bigy = y
	assert(container[1], "horizontal called without arguments!")
	for i, v in ipairs(container) do
		local ret = {}
		ret.x, ret.y = layout.handle(v,x,y, simulate)
		if ret.y > bigy then bigy = ret.y end
		x = ret.x
	end
	return x,bigy
end


function layout.copySize(container, x, y, simulate)
	local bigx, bigy = layout.handle(container[2], x, y, true)
	container[1].width = bigx -x
	container[1].height = bigy -y
	assert(container[1])
	assert(container[2])
	layout.handle(container[1], x, y, simulate)
	layout.handle(container[2], x, y, simulate)
	return bigx, bigy
end


function layout.vertical(container, x, y, simulate)
	local bigx = x
	for i, v in ipairs(container) do
		local ret = {}
		ret.x, ret.y = layout.handle(v,x,y, simulate)
		if ret.x > bigx then bigx = ret.x end
		y = ret.y
	end
	return bigx,y
end


local depth = -1
function layout.handle(container, x, y, simulate)
	assert_num("layout.handle", 2, x, y)
	assert(container, "layout.handle called without container")
	name = container["name"]
	assert(name, "layout.handle container without name")
	assert(layout[name], "Unknown layout function " .. name .. " called")
	depth = depth +1
	local depthCursor = ""
	for i=0, depth do
		depthCursor = depthCursor .. ".."
	end
	if depth == 0 then
		if simulate then
			debugPrint("Starting simulation")
		else
			debugPrint("Starting rendering")
		end
	end
	debugPrint(depthCursor.. ">" .. container.name)
	x, y = layout[name](container, x, y, simulate)
	assert_num("layout.handle" .. container.name .. " is not returning properly!", 2, x, y)
	debugPrint(depthCursor.."<" .. container.name)
	depth = depth -1
	if depth == -1 then
		if simulate then
			debugPrint("simulation ended\n")
		else
			debugPrint("rendering ended\n")
		end
	end
	return x, y
end
